# encoding: utf-8

# -------------------------------------------------------------------------------
# (C) Copyright 2019 - 2021 Micro Focus or one of its affiliates.
#
# The only warranties for products and services of Micro Focus and its affiliates
# and licensors (“Micro Focus”) are as may be set forth in the express warranty
# statements accompanying such products and services. Nothing herein should be
# construed as constituting an additional warranty. Micro Focus shall not be
# liable for technical or editorial errors or omissions contained herein. The
# information contained herein is subject to change without notice.
#
# Except as specifically indicated otherwise, this document contains confidential
# information and a valid license is required for possession, use or copying. If
# this work is provided to the U.S. Government, consistent with FAR 12.211 and
# 12.212, Commercial Computer Software, Computer Software Documentation, and
# Technical Data for Commercial Items  are licensed to the U.S. Government under
# vendor's standard commercial license.
# -------------------------------------------------------------------------------

# File:        modules/Iprint.ycp
# Package:     Configuration of iprint
# Summary:     Iprint settings, input and output functions
# Authors:     Mark Clark <mclark@novell.com>
#
# $Id: Iprint.ycp,v 1.1.2.62 2005/11/11 19:00:34 mclark Exp $
#
# Representation of the configuration of iprint.
# Input and output routines.
require "yast"

module Yast
  class IprintClass < Module
    def main
      textdomain "novell-iprint"

      Yast.import "Popup"
      Yast.import "Service"
      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"
      Yast.import "Edirectory"
      Yast.import "EdirUtils"
      Yast.import "Mode"
      Yast.import "Stage"
      Yast.import "NovellUtils"
      Yast.import "NovellLDAP"
      Yast.import "NovellPatterns"
      Yast.import "NovellAddonInstall"
      Yast.import "Arch"
      Yast.import "Wizard"

      # Settings Map
      @Settings = {}
      @oldSettings = {} # OES 1.0 or 2.0 settings for upgrade
      @auto_clone = false
      @xad_packages_installed = false
      @xadMode = false

      @xadTestMode = false

      @edir_service_configured = false

      @schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"

      # Data was modified?
      @modified = false


      @proposal_valid = false

      @upgrade_from = ""

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false
      @globals_configured = false
      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    def GetNameMap
      nameMap = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        nameMap = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_CONVERT_EXISTING_CONTAINER"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "iPrint.GetNameMap (post install):are we name mapped from xad conf file:%1",
          nameMap
        )
      else
        nameMap = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_CONVERT_EXISTING_CONTAINER",
          ""
        )
        Builtins.y2milestone(
          "iPrint.GetNameMap:are we name mapped from eDir map:%1",
          nameMap
        )
        if nameMap == nil || nameMap == ""
          nameMap = Convert.to_string(
            SCR.Read(
              path(".sysconfig.novell.xad.XAD_CONVERT_EXISTING_CONTAINER")
            )
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "iPrint.GetNameMap:are we name mapped from xad conf file:%1",
            nameMap
          )
        end
      end
      nameMap = "no" if nameMap == nil || nameMap == ""

      nameMap
    end

    def GetNameMapContainer
      nameMapContainer = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        nameMapContainer = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_EXISTING_CONTAINER"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "iPrint.GetNameMapContainer (post install):name mapped container from xad conf file:%1",
          nameMapContainer
        )
      else
        nameMapContainer = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_EXISTING_CONTAINER",
          ""
        )
        Builtins.y2milestone(
          "iPrint.GetNameMapContainer:name mapped container from eDir map:%1",
          nameMapContainer
        )
        if nameMapContainer == nil || nameMapContainer == ""
          nameMapContainer = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_EXISTING_CONTAINER"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "iPrint.GetNameMapContainer:name mapped container from xad conf file:%1",
            nameMapContainer
          )
        end
      end
      if nameMapContainer == nil || nameMapContainer == ""
        nameMapContainer = ""
      else
        nameMapContainer = NovellUtils.Nds2Ldap(nameMapContainer)
      end

      nameMapContainer
    end

    def GetDomainName
      domainName = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        domainName = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_NAME"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "iPrint.GetDomainName (post install): domain name from xad conf file:%1",
          domainName
        )
      else
        domainName = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_DOMAIN_NAME",
          ""
        )
        Builtins.y2milestone(
          "iPrint.GetDomainName:domain name from eDir map:%1",
          domainName
        )
        if domainName == nil || domainName == ""
          domainName = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_NAME"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "iPrint.GetDomainName: domain name from xad conf file:%1",
            domainName
          )
        end
      end
      if domainName == nil || domainName == ""
        tmpMap = Edirectory.ProposeXadSettings # worst case, propose the system domain name.
        domainName = Ops.get_string(tmpMap, "XAD_DOMAIN_NAME", "")
        Builtins.y2milestone(
          "iPrint.GetDomainName:domain name from system domain name:%1",
          domainName
        )
      end
      domainName
    end

    def GetXadDomainType
      domainType = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        domainType = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_TYPE"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "iPrint.GetXadUnixObjects (post install):domain type from xad conf file:%1",
          domainType
        )
      else
        domainType = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_DOMAIN_TYPE",
          ""
        )
        Builtins.y2milestone(
          "iPring.GetXadObjects:domain type from eDir map:%1",
          domainType
        )
        if domainType == nil || domainType == ""
          domainType = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_DOMAIN_TYPE"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "iPrint.GetXadUnixObjects:domain type from xad conf file:%1",
            domainType
          )
        end
      end
      domainType = "" if domainType == nil

      domainType
    end

    def GetTreeType
      treeType = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        treeType = Convert.to_string(
          SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
        ) # next try reading the edir conf file
        Builtins.y2milestone(
          "iPrint.GetTreeType (post install):tree type from edir conf file:%1",
          treeType
        )
      else
        treeType = Ops.get_string(
          Edirectory.EdirSettings,
          "CONFIG_EDIR_TREE_TYPE",
          ""
        )
        Builtins.y2milestone(
          "iPrint.GetTreeType:tree type from eDir map:%1",
          treeType
        )
        if treeType == nil || treeType == ""
          treeType = Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_TREE_TYPE"))
          ) # next try reading the edir conf file
          Builtins.y2milestone(
            "iPrint.GetTreeType:tree type from edir conf file:%1",
            treeType
          )
        end
      end
      treeType = "" if treeType == nil

      treeType
    end

    def GetServerContext
      serverContext = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        serverContext = Convert.to_string(
          SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"))
        )
        Builtins.y2milestone(
          "iPrint.GetServerContext (post install mode):server context from eDir file:%1",
          serverContext
        )
      else
        serverContext = Ops.get_string(
          Edirectory.EdirSettings,
          "CONFIG_EDIR_SERVER_CONTEXT",
          ""
        )
        Builtins.y2milestone(
          "iPrint.GetServerContext:server context from eDir map:%1",
          serverContext
        )
        if serverContext == nil || serverContext == ""
          serverContext = Convert.to_string(
            SCR.Read(path(".sysconfig.edirectory.CONFIG_EDIR_SERVER_CONTEXT"))
          ) # next try reading the edir conf file
          Builtins.y2milestone(
            "iPrint.GetServerContext:server context from edir conf file:%1",
            serverContext
          )
        end
      end

      serverContext = "" if serverContext == nil

      serverContext
    end

    def GetXadForestRoot
      domainName = GetDomainName()

      forestRootDomain = ""
      if Mode.installation && Stage.normal && @edir_service_configured
        forestRootDomain = Convert.to_string(
          SCR.Read(path(".sysconfig.novell.xad.XAD_FOREST_ROOT"))
        ) # next try reading the xad conf file
        Builtins.y2milestone(
          "iPrint.GetXadForestRoot(post install):forest root from xad conf file:%1",
          forestRootDomain
        )
      else
        forestRootDomain = Ops.get_string(
          Edirectory.EdirSettings,
          "XAD_FOREST_ROOT",
          ""
        )
        Builtins.y2milestone(
          "iPrint.GetXadForestRoot:forest root from eDir map:%1",
          forestRootDomain
        )
        if forestRootDomain == nil || forestRootDomain == ""
          forestRootDomain = Convert.to_string(
            SCR.Read(path(".sysconfig.novell.xad.XAD_FOREST_ROOT"))
          ) # next try reading the xad conf file
          Builtins.y2milestone(
            "iPrint.GetXadForestRoot:forest root from xad conf file:%1",
            forestRootDomain
          )
        end
      end
      forestRootDomain = "" if forestRootDomain == nil

      treeType = GetTreeType()
      domainType = GetXadDomainType()

      if domainType == "forest" && treeType == "new"
        if Builtins.toupper(domainName) != Builtins.toupper(forestRootDomain)
          forestRootDomain = domainName
        end
      end
      forestRootDomain
    end

    def extend_schema
    # determine IP addresses
      server_list = Ops.get_list(
        NovellLDAP.Settings,
        "CONFIG_LDAP_SERVERS_LIST",
        []
        )
      primary_server = NovellLDAP.GetAddressServer(
      Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", ""),
       server_list
      )
      fallback_list = NovellLDAP.GetFallbackServerList(
        Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", ""),
        server_list
      )
      Builtins.y2milestone("iPrint:Primary server: %1", primary_server)
      Builtins.y2milestone("iPrint:Fallback server_list: %1", fallback_list)
      full_admin_context = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_CONTEXT",
        ""
      )
      admin_pwd = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_ADMIN_PASSWORD",
        ""
      )
      treeName = Ops.get_string(
        NovellLDAP.Settings,
        "CONFIG_LDAP_TREE_NAME",
        ""
       )
      ldapSecurePort = Builtins.tostring(
        Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
      )
      apache_https_port = "443"

      full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)
      Builtins.y2milestone("iPrint:admin:%1", full_admin_context)
      if @xadMode
        xad_tree_admin_context = Ops.get_string(
        NovellLDAP.Settings,
        "XAD_TREE_ADMIN_CONTEXT",
        ""
        )
        Builtins.y2milestone(
          "iPrint:XAD_TREE_ADMIN_CONTEXT:%1",
          xad_tree_admin_context
        )
        if xad_tree_admin_context != nil && xad_tree_admin_context != ""
          xad_tree_admin_context = NovellUtils.Ldap2Nds(
          xad_tree_admin_context
          )
          if xad_tree_admin_context != full_admin_context
            full_admin_context = xad_tree_admin_context
            admin_pwd = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_PASSWORD",
              ""
            )
            Builtins.y2milestone("iPrint:admin:%1", full_admin_context)
          end
        end
      end
      upgradeStr = ""
      if Mode.update || Mode.autoupgrade
        upgradeStr = "UPGRADE='1' "
      end

      # Extend iPrint schema.
      insufficient_rights = false

      full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)

      schema_file = "/opt/novell/sch/iprint.sch"
      ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
      cmd2 = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(ndssch_file, " -h "),
                        Ops.get_string(
                          primary_server,
                          "CONFIG_LDAP_SERVER_IP",
                          ""
                        )
                      ),
                      " -F '"
                    ),
                    @schemaLogPath
                  ),
                  "' '"
                ),
                full_admin_context
              ),
              "'"
            ),
            " "
          ),
          schema_file
        )
      Builtins.y2milestone("iPrint:Executing %1", cmd2)
      rmap = Convert.to_map(
        SCR.Execute(
          path(".target.bash_output"),
          Ops.add(
            Ops.add(
              Ops.add(Ops.add(upgradeStr, "ADM_PASSWD='"), admin_pwd),
              "' "
            ),
            cmd2
          )
        )
      )
      if Ops.get_integer(rmap, "exit", -1) != 0
        Builtins.foreach(fallback_list) do |server_info|
          cmd2 = Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(ndssch_file, " -h "),
                          Ops.get_string(
                            server_info,
                            "CONFIG_LDAP_SERVER_IP",
                            ""
                          )
                        ),
                        " -F '"
                      ),
                      @schemaLogPath
                    ),
                    "' '"
                  ),
                  full_admin_context
                ),
                "'"
              ),
              " "
            ),
            schema_file
          )
          Builtins.y2milestone(
              "iPrint:Retry of schema extension with fallback IP address / hostname:Executing %1",
              cmd2
            )
          rmap = Convert.to_map(
            SCR.Execute(
              path(".target.bash_output"),
              Ops.add(
                Ops.add(
                  Ops.add(Ops.add(upgradeStr, "ADM_PASSWD='"), admin_pwd),
                  "' "
                ),
                cmd2
              )
            )
          )
          if Ops.get_integer(rmap, "exit", -1) != 0
            Builtins.y2milestone(
              "iPrint:extend schema(sch) failed: %1 using file %2",
              Ops.get_integer(rmap, "exit", -1),
              schema_file
            )
          else
            raise Break
          end
        end
      end
    end
    # Read all iprint settings
    # @return true on success
    def Read
      # Iprint read dialog caption
      caption = _("Initializing iPrint Configuration")

      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/1
          _("Read iPrint settings"),
          _("Checking dependencies")
        ],
        [
          # Progress step 1/1
          _("Reading iPrint settings..."),
          _("Checking dependencies"),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read current settings
      return false if Abort()
      Progress.NextStage

      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )
      Builtins.y2milestone(
        "iPrint:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end

      @xadMode = true if @xadTestMode

      Builtins.y2milestone("iPrint:xadMode:%1", @xadMode)

      # Test for the upgrade scenario
      @upgrade_from = NovellUtils.UpgradeIsFrom
      if (Mode.update || Mode.autoupgrade) &&  @upgrade_from != ""
        @oldSettings = NovellUtils.GetOldSysconfigMap("iprint")
      end

      Builtins.y2milestone(
        "iPrint:,
        upgradeFrom = #{@upgrade_from}"
      )
      Builtins.y2milestone("iPrint:oldSettings: %1", @oldSettings)

      info = NovellAddonInstall.GetProductInfo("edirectory")
      if Ops.get_boolean(info, "SERVICE_CONFIGURED", false) == true
        @edir_service_configured = true
      else
        @edir_service_configured = false
      end

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Read
      Progress.set(progress_orig)

      # determine if iPrint configuration already exists
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.iprint.SERVICE_CONFIGURED"))
      )
      service_configured = Builtins.tolower(service_configured)
      if Mode.update || Mode.autoupgrade
        # Existing IP address
        existingIP = Ops.get_string(
          @oldSettings,
          "CONFIG_IPRINT_LDAP_SERVER",
          ""
        )
        existingIP = "" if existingIP == nil
        Ops.set(@Settings, "CONFIG_IPRINT_LDAP_SERVER", existingIP)
        admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          ""
        )
        admin_context = NovellUtils.Ldap2Nds(admin_context)
        Builtins.y2milestone(
          "iPrint.Read (upgrade mode oes2):admin:%1",
          admin_context
        )
        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "iPrint.Read:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            xad_tree_admin_context = NovellUtils.Ldap2Nds(
              xad_tree_admin_context
            )
            if xad_tree_admin_context != admin_context
              admin_context = xad_tree_admin_context
              Builtins.y2milestone(
                "iPrint.Read:admin (upgrade mode oes2):%1",
                admin_context
              )
            end
          end
        end
        admin_context = NovellUtils.Ldap2Nds(admin_context)
        # string topContainer = NovellUtils::GetTopContextNDS( admin_context );
        # y2milestone("iPrint.Read:Top Container (upgrade mode oes2): %1",topContainer );
        # Settings["CONFIG_IPRINT_TOP_CONTEXT"] = topContainer;
        Ops.set(
          @Settings,
          "CONFIG_IPRINT_TOP_CONTEXT",
          Ops.get_string(@oldSettings, "CONFIG_IPRINT_TOP_CONTEXT", "")
        )
      # normal non-upgrade install mode
      else
        if service_configured != "yes"
          server_list = NovellLDAP.GetLocalServers(nil)
          if Ops.greater_than(Builtins.size(server_list), 0)
            Ops.set(
              @Settings,
              "CONFIG_IPRINT_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          else
            server_list = NovellLDAP.GetRemoteServers(nil)
            Ops.set(
              @Settings,
              "CONFIG_IPRINT_LDAP_SERVER",
              Ops.get_string(
                Ops.get(server_list, 0, {}),
                "CONFIG_LDAP_SERVER_IP",
                ""
              )
            )
          end

          full_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "CONFIG_LDAP_ADMIN_CONTEXT",
            ""
          )
          full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)
          Builtins.y2milestone("iPrint.Read:admin:%1", full_admin_context)
          if @xadMode
            xad_tree_admin_context = Ops.get_string(
              NovellLDAP.Settings,
              "XAD_TREE_ADMIN_CONTEXT",
              ""
            )
            Builtins.y2milestone(
              "iPrint.Read:XAD_TREE_ADMIN_CONTEXT:%1",
              xad_tree_admin_context
            )
            if xad_tree_admin_context != nil && xad_tree_admin_context != ""
              xad_tree_admin_context = NovellUtils.Ldap2Nds(
                xad_tree_admin_context
              )
              if xad_tree_admin_context != full_admin_context
                full_admin_context = xad_tree_admin_context
                Builtins.y2milestone("iPrint.Read:admin:%1", full_admin_context)
              end
            end
          end
          full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)
          topContainer = NovellUtils.GetTopContextNDS(full_admin_context)
          Builtins.y2milestone("iPrint.Read:Top Container: %1", topContainer)
          Ops.set(@Settings, "CONFIG_IPRINT_TOP_CONTEXT", topContainer) # the config file exists, so use the data.
        else
          if Mode.normal
            name = "iPrint"
            if !Popup.YesNo(
                Builtins.sformat(
                  _(
                    "It appears that %1 has already been configured.  To reconfigure %2 please exit and use iManager. \nIf %3 has been uninstalled and is being reinstalled you may continue.  Continue configuration?"
                  ),
                  name,
                  name,
                  name
                )
              )
              return false
            end #return (any)false;
          end

          Ops.set(
            @Settings,
            "CONFIG_IPRINT_LDAP_SERVER",
            Convert.to_string(
              SCR.Read(path(".sysconfig.iprint.CONFIG_IPRINT_LDAP_SERVER"))
            )
          )
          Ops.set(
            @Settings,
            "CONFIG_IPRINT_TOP_CONTEXT",
            Convert.to_string(
              SCR.Read(path(".sysconfig.iprint.CONFIG_IPRINT_TOP_CONTEXT"))
            )
          )
        end
      end

      @globals_configured = true
      Builtins.y2debug("IPRINT READ Complete: %1", @Settings)

      # Error message
      Report.Error(_("Cannot read current iPrint settings.")) if false
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end

    # insufficient_rights_error()
    #
    # Checks a string for output indicating insufficient rights.
    def insufficient_rights_error(msg)
      return false if msg == nil || msg == ""

      idx = Builtins.search(msg, "Insufficient access")
      return true if idx != nil && Ops.greater_or_equal(idx, 0)

      idx = Builtins.search(msg, "no access (-672)")
      return true if idx != nil && Ops.greater_or_equal(idx, 0)

      false
    end

    # Write all iprint settings
    # @return true on success
    def Write
      sl = 500

      Wizard.DisableAbortButton

      pattern = "novell-iprint"
      if NovellPatterns.IsPatternInstalled(pattern) == false
        Builtins.y2milestone(
          "The required pattern, %1, is not installed.",
          pattern
        )
        if Mode.normal
          Report.Error(
            Builtins.sformat(
              _(
                "Configuration aborted. Install the required %1 pattern and try again."
              ),
              pattern
            )
          )
        end

        return true
      end

      abortInstall = Builtins.tolower(
        NovellUtils.ReadOption("ABORT_OES_INSTALL")
      )
      if abortInstall == "yes"
        Builtins.y2milestone(
          "iPrint:setting ABORT_OES_INSTALL is set to 'yes'."
        )
        return true
      end
      # See if XAD is installed. If so, default the data.
      @xad_packages_installed = NovellPatterns.IsPatternInstalled(
        "novell-domain-services"
      )
      Builtins.y2milestone(
        "iPrint:xad_packages_installed:%1",
        @xad_packages_installed
      )
      if @xad_packages_installed
        if Ops.get_string(Edirectory.EdirSettings, "XAD_SERVICE_CONFIGURED", "") == "yes" ||
            Convert.to_string(
              SCR.Read(path(".sysconfig.novell.xad.SERVICE_CONFIGURED"))
            ) == "yes"
          @xadMode = true
        end
      end

      @xadMode = true if @xadTestMode

      Builtins.y2milestone("iPrint:xadMode:%1", @xadMode)

      if !Mode.update && !Mode.autoupgrade
        # Iprint read dialog caption
        if !Ops.get(NovellAddonInstall.products_to_configure, "iprint", true)
          Builtins.y2milestone("iPrint -- configure later")
          return true
        end

        # If Aquila did not install properly, then we don't want to proceed at this point.
        Builtins.y2milestone(
          "iPrint:testing to see if eDir configured properly. Xad mode: %1",
          @xadMode
        )
        if @xadMode == true
          Builtins.y2milestone(
            "iPrint:eDir service_configured (mem):%1",
            Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "")
          )
          Builtins.y2milestone(
            "iPrint:eDir service_configured (file):%1",
            Convert.to_string(
              SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
            )
          )
          if Ops.get_string(Edirectory.EdirSettings, "SERVICE_CONFIGURED", "") != "yes" &&
              Convert.to_string(
                SCR.Read(path(".sysconfig.edirectory.SERVICE_CONFIGURED"))
              ) != "yes"
            if EdirUtils.EdirConfigProblemPrompt("iPrint")
              Builtins.y2milestone(
                "iPrint:aborting iPrint write by user choice since eDirectory was not configured."
              )
              return true
            end
          end
        end

        caption = _("Saving iPrint Configuration")

        steps = 1

        Builtins.sleep(sl)

        # We do not set help text here, because it was set outside
        Progress.New(
          caption,
          " ",
          steps,
          [
            # Progress stage 1/1
            _("Write iPrint settings")
          ],
          [
            # Progress step 1/1
            _("Writing iPrint settings..."),
            # Progress finished
            _("Finished")
          ],
          ""
        )

        # write settings
        return false if Abort()
        Progress.NextStage

        SCR.Write(path(".sysconfig.iprint.SERVICE_CONFIGURED"), "no")

        # apply settings -- Note, eDirectory must be up and running for everything to work.  We will assume
        # that the eDirectory configure will not exit until ndsd is up and running.  That way, we can avoid
        # coding in the delay loop timing kludge that exists in the python code.


        # determine IP addresses
        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", ""),
          server_list
        )
        fallback_list = NovellLDAP.GetFallbackServerList(
          Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", ""),
          server_list
        )
        Builtins.y2milestone("iPrint:Primary server: %1", primary_server)
        Builtins.y2milestone("iPrint:Fallback server_list: %1", fallback_list)
        full_admin_context = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          ""
        )
        admin_pwd = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_PASSWORD",
          ""
        )
        treeName = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_TREE_NAME",
          ""
        )
        ldapSecurePort = Builtins.tostring(
          Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
        )
        apache_https_port = "443"

        full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)
        Builtins.y2milestone("iPrint:admin:%1", full_admin_context)
        if @xadMode
          xad_tree_admin_context = Ops.get_string(
            NovellLDAP.Settings,
            "XAD_TREE_ADMIN_CONTEXT",
            ""
          )
          Builtins.y2milestone(
            "iPrint:XAD_TREE_ADMIN_CONTEXT:%1",
            xad_tree_admin_context
          )
          if xad_tree_admin_context != nil && xad_tree_admin_context != ""
            xad_tree_admin_context = NovellUtils.Ldap2Nds(
              xad_tree_admin_context
            )
            if xad_tree_admin_context != full_admin_context
              full_admin_context = xad_tree_admin_context
              admin_pwd = Ops.get_string(
                NovellLDAP.Settings,
                "XAD_TREE_ADMIN_PASSWORD",
                ""
              )
              Builtins.y2milestone("iPrint:admin:%1", full_admin_context)
            end
          end
        end

        # Extend iPrint schema.
        insufficient_rights = false

        upgradeStr = ""
        if Mode.update || Mode.autoupgrade
          upgradeStr = "UPGRADE='1' "
        end

        full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)

        schema_file = "/opt/novell/sch/iprint.sch"
        ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
        cmd2 = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(ndssch_file, " -h "),
                        Ops.get_string(
                          primary_server,
                          "CONFIG_LDAP_SERVER_IP",
                          ""
                        )
                      ),
                      " -F '"
                    ),
                    @schemaLogPath
                  ),
                  "' '"
                ),
                full_admin_context
              ),
              "'"
            ),
            " "
          ),
          schema_file
        )
        Builtins.y2milestone("iPrint:Executing %1", cmd2)
        rmap = Convert.to_map(
          SCR.Execute(
            path(".target.bash_output"),
            Ops.add(
              Ops.add(
                Ops.add(Ops.add(upgradeStr, "ADM_PASSWD='"), admin_pwd),
                "' "
              ),
              cmd2
            )
          )
        )
        if Ops.get_integer(rmap, "exit", -1) != 0
          Builtins.foreach(fallback_list) do |server_info|
            cmd2 = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(ndssch_file, " -h "),
                            Ops.get_string(
                              server_info,
                              "CONFIG_LDAP_SERVER_IP",
                              ""
                            )
                          ),
                          " -F '"
                        ),
                        @schemaLogPath
                      ),
                      "' '"
                    ),
                    full_admin_context
                  ),
                  "'"
                ),
                " "
              ),
              schema_file
            )
            Builtins.y2milestone(
              "iPrint:Retry of schema extension with fallback IP address / hostname:Executing %1",
              cmd2
            )
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(
                  Ops.add(
                    Ops.add(Ops.add(upgradeStr, "ADM_PASSWD='"), admin_pwd),
                    "' "
                  ),
                  cmd2
                )
              )
            )
            if Ops.get_integer(rmap, "exit", -1) != 0
              Builtins.y2milestone(
                "iPrint:extend schema(sch) failed: %1 using file %2",
                Ops.get_integer(rmap, "exit", -1),
                schema_file
              )
            else
              raise Break
            end
          end
        end

        return false if Abort()

        # Create the iprint_ssl.conf file. The RPM installs a template version which we will write over.
        treeName = Builtins.toupper(treeName)
        tree_name_entry = Builtins.sformat(
          "        AuthName \"%1\"\n",
          treeName
        )

        # Get the DNS name of the machine we are on.
        dnsName = NovellUtils.GetDNSname

        topContainer = Ops.get_string(
          @Settings,
          "CONFIG_IPRINT_TOP_CONTEXT",
          ""
        )
        topContainer = "" if topContainer == nil
        Builtins.y2milestone("iPrint:Top Container: %1", topContainer)

        portString = Ops.add(":", ldapSecurePort)
        portString = ":1636" if @xadMode

        dnsName = Ops.add(dnsName, portString)

        lib = "lib" # this is the 32 bit version
        lib = "lib64" if Arch.x86_64 # this is the 64 bit path


        ldap_url_entry = Builtins.sformat(
          "        AuthLDAPDNURL \"ldaps://%1/%2???(objectClass=user)\"\n",
          dnsName,
          topContainer
        )

        iprint_ssl_conf = Ops.add(
          Ops.add(
            Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              "# Do not modify this file!  Changes to this file may\n" +
                                "# cause iPrint to opperate improperly.  Your changes to this\n" +
                                "# file will not be preserved.\n\n" +
                                "<IfModule !mod_ipp.c>\n" +
                                "   LoadModule ipp_module /opt/novell/iprint/httpd/",
                              lib
                            ),
                            "/mod_ipp.so\n"
                          ),
                          "</IfModule>\n\n"
                        ),
                        "<IfModule mod_ipp.c>\n"
                      ),
                      "      <LocationMatch ^/(ipps|idss|[Pp][Ss][Mm][Ss][Tt][Aa][Tt][Uu][Ss])($|/)>\n"
                    ),
                    "        SSLRequireSSL\n"
                  ),
                  "        Require valid-user\n"
                ),
                " #       Require all granted\n"
              ),
            "        AuthType Basic\n"
          ),
          "        AuthBasicProvider ldapdn\n"
        )

        iprint_ssl_conf_2 = "        AuthLDAPDNRemoteUserIsDN on\n" +
          "        AuthzLDAPDNAuthoritative Off\n" +
          "        AuthLDAPDNAllowDNAuth on\n" +
          "      </LocationMatch>\n" + "</IfModule>\n"

        ret = false
        # Start out with a blank file.
        ret = SCR.Write(
          path(".target.string"),
          "/etc/opt/novell/iprint/httpd/conf/iprint_ssl.conf",
          ""
        )
        ret = SCR.Write(
          path(".target.string"),
          "/etc/opt/novell/iprint/httpd/conf/iprint_ssl.conf",
          Ops.add(
            Ops.add(Ops.add(iprint_ssl_conf, tree_name_entry), ldap_url_entry),
            iprint_ssl_conf_2
          )
        )
        if ret != true
          Report.Warning(
            _(
              "Failed to write to /etc/opt/novell/iprint/httpd/conf/iprint_ssl.conf"
            )
          )
        end

        iprint_g_conf = Ops.add(
          Ops.add(
            Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(
                            Ops.add(
                              Ops.add(
                                Ops.add(
                                  Ops.add(
                                    Ops.add(
                                      Ops.add(
                                        Ops.add(
                                          Ops.add(
                                            Ops.add(
                                              Ops.add(
                                                Ops.add(
                                                  Ops.add(
                                                    Ops.add(
                                                      Ops.add(
                                                        Ops.add(
                                                          Ops.add(
                                                            Ops.add(
                                                              Ops.add(
                                                                Ops.add(
                                                                  Ops.add(
                                                                    Ops.add(
                                                                      Ops.add(
                                                                        Ops.add(
                                                                          Ops.add(
                                                                            Ops.add(
                                                                              Ops.add(
                                                                                Ops.add(
                                                                                  Ops.add(
                                                                                    Ops.add(
                                                                                      Ops.add(
                                                                                        Ops.add(
                                                                                          Ops.add(
                                                                                            Ops.add(
                                                                                              Ops.add(
                                                                                                Ops.add(
                                                                                                  Ops.add(
                                                                                                    Ops.add(
                                                                                                      Ops.add(
                                                                                                        Ops.add(
                                                                                                          Ops.add(
                                                                                                            Ops.add(
                                                                                                              Ops.add(
                                                                                                                Ops.add(
                                                                                                                  Ops.add(
                                                                                                                    Ops.add(
                                                                                                                      Ops.add(
                                                                                                                        Ops.add(
                                                                                                                          Ops.add(
                                                                                                                            Ops.add(
                                                                                                                              Ops.add(
                                                                                                                                Ops.add(
                                                                                                                                  Ops.add(
                                                                                                                                    Ops.add(
                                                                                                                                      Ops.add(
                                                                                                                                        Ops.add(
                                                                                                                                          Ops.add(
                                                                                                                                            Ops.add(
                                                                                                                                              Ops.add(
                                                                                                                                                Ops.add(
                                                                                                                                                  Ops.add(
                                                                                                                                                    Ops.add(
                                                                                                                                                      Ops.add(
                                                                                                                                                        Ops.add(
                                                                                                                                                          Ops.add(
                                                                                                                                                            Ops.add(
                                                                                                                                                              Ops.add(
                                                                                                                                                                Ops.add(
                                                                                                                                                                  Ops.add(
                                                                                                                                                                    Ops.add(
                                                                                                                                                                      Ops.add(
                                                                                                                                                                        Ops.add(
                                                                                                                                                                          Ops.add(
                                                                                                                                                                            Ops.add(
                                                                                                                                                                              Ops.add(
                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                Ops.add(
                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                                Ops.add(
                                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                                                                Ops.add(
                                                                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                                                                                Ops.add(
                                                                                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                                                                                                Ops.add(
                                                                                                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                                                                                                              Ops.add(
                                                                                                                                                                                                                                                                                                Ops.add(
                                                                                                                                                                                                                                                                                                  Ops.add(
                                                                                                                                                                                                                                                                                                    Ops.add(
                                                                                                                                                                                                                                                                                                      Ops.add(
                                                                                                                                                                                                                                                                                                        Ops.add(
                                                                                                                                                                                                                                                                                                          Ops.add(
                                                                                                                                                                                                                                                                                                            Ops.add(
                                                                                                                                                                                                                                                                                                              "# Do not modify this file with the exception of\n" +
                                                                                                                                                                                                                                                                                                                "# the ServerName directive below. Other changes to this\n" +
                                                                                                                                                                                                                                                                                                                "# file will not be preserved. Changes to this file may\n" +
                                                                                                                                                                                                                                                                                                                "# cause iPrint to function improperly.\n" + "\nListen 631\n\n" +
                                                                                                                                                                                                                                                                                                                "<IfModule !mod_ipp.c>\n" +
                                                                                                                                                                                                                                                                                                                "   LoadModule ipp_module /opt/novell/iprint/httpd/",
                                                                                                                                                                                                                                                                                                              lib
                                                                                                                                                                                                                                                                                                            ),
                                                                                                                                                                                                                                                                                                            "/mod_ipp.so\n"
                                                                                                                                                                                                                                                                                                          ),
                                                                                                                                                                                                                                                                                                          "</IfModule>\n\n"
                                                                                                                                                                                                                                                                                                        ),
                                                                                                                                                                                                                                                                                                        "<IfModule !util_ldapdn.c>\n"
                                                                                                                                                                                                                                                                                                      ),
                                                                                                                                                                                                                                                                                                      "   LoadModule ldapdn_module       /opt/novell/iprint/httpd/"
                                                                                                                                                                                                                                                                                                    ),
                                                                                                                                                                                                                                                                                                    lib
                                                                                                                                                                                                                                                                                                  ),
                                                                                                                                                                                                                                                                                                  "/mod_ldapdn.so\n"
                                                                                                                                                                                                                                                                                                ),
                                                                                                                                                                                                                                                                                                "</IfModule>\n\n"
                                                                                                                                                                                                                                                                                              ),
                                                                                                                                                                                                                                                                                              "<IfModule !mod_authnz_ldapdn.c>\n"
                                                                                                                                                                                                                                                                                            ),
                                                                                                                                                                                                                                                                                            "   LoadModule authnz_ldapdn_module /opt/novell/iprint/httpd/"
                                                                                                                                                                                                                                                                                          ),
                                                                                                                                                                                                                                                                                          lib
                                                                                                                                                                                                                                                                                        ),
                                                                                                                                                                                                                                                                                        "/mod_authnz_ldapdn.so\n"
                                                                                                                                                                                                                                                                                      ),
                                                                                                                                                                                                                                                                                      "</IfModule>\n\n"
                                                                                                                                                                                                                                                                                    ),
                                                                                                                                                                                                                                                                                    "<IFModule mod_authnz_ldapdn.c>\n"
                                                                                                                                                                                                                                                                                  ),
                                                                                                                                                                                                                                                                                  "   LDAPDNTrustedGlobalCert CA_BASE64 /etc/opt/novell/certs/SSCert.pem\n"
                                                                                                                                                                                                                                                                                ),
                                                                                                                                                                                                                                                                                "</IfModule>\n\n"
                                                                                                                                                                                                                                                                              ),
                                                                                                                                                                                                                                                                              "<LocationMatch /(ids|ipp)>\n"
                                                                                                                                                                                                                                                                            ),
                                                                                                                                                                                                                                                                            "  Require all granted\n"
                                                                                                                                                                                                                                                                          ),
                                                                                                                                                                                                                                                                        "</LocationMatch>\n\n"
                                                                                                                                                                                                                                                                      ),
                                                                                                                                                                                                                                                                      "<IfModule !mod_expires.c>\n"
                                                                                                                                                                                                                                                                    ),
                                                                                                                                                                                                                                                                    "   LoadModule expires_module /usr/"
                                                                                                                                                                                                                                                                  ),
                                                                                                                                                                                                                                                                  lib
                                                                                                                                                                                                                                                                ),
                                                                                                                                                                                                                                                                "/apache2-worker/mod_expires.so\n"
                                                                                                                                                                                                                                                              ),
                                                                                                                                                                                                                                                              "</IfModule>\n"
                                                                                                                                                                                                                                                            ),
                                                                                                                                                                                                                                                            "<IfModule !mod_headers.c>\n"
                                                                                                                                                                                                                                                          ),
                                                                                                                                                                                                                                                          "   LoadModule headers_module /usr/"
                                                                                                                                                                                                                                                        ),
                                                                                                                                                                                                                                                        lib
                                                                                                                                                                                                                                                      ),
                                                                                                                                                                                                                                                      "/apache2-worker/mod_headers.so\n"
                                                                                                                                                                                                                                                    ),
                                                                                                                                                                                                                                                    "</IfModule>\n\n"
                                                                                                                                                                                                                                                  ),
                                                                                                                                                                                                                                                  "<IfModule headers_module>\n"
                                                                                                                                                                                                                                                ),
                                                                                                                                                                                                                                                "   Header add X-Frame-Options SAMEORIGIN\n"
                                                                                                                                                                                                                                              ),
                                                                                                                                                                                                                                              "</IfModule>\n\n"
                                                                                                                                                                                                                                            ),
                                                                                                                                                                                                                                            "# Set up our ipp docs directory\n"
                                                                                                                                                                                                                                          ),
                                                                                                                                                                                                                                          "Alias /ippdocs/ \"/var/opt/novell/iprint/htdocs/\"\n"
                                                                                                                                                                                                                                        ),
                                                                                                                                                                                                                                        "<Directory /var/opt/novell/iprint/htdocs>\n"
                                                                                                                                                                                                                                      ),
                                                                                                                                                                                                                                      "   Options FollowSymLinks MultiViews\n"
                                                                                                                                                                                                                                    ),
                                                                                                                                                                                                                                    "   AllowOverride None\n"
                                                                                                                                                                                                                                  ),
                                                                                                                                                                                                                                  "   Require all granted\n"
                                                                                                                                                                                                                                ),
                                                                                                                                                                                                                              "   AddType application/x-rpm .rpm\n"
                                                                                                                                                                                                                            ),
                                                                                                                                                                                                                            "   AddType application/octet-stream dmg\n"
                                                                                                                                                                                                                          ),
                                                                                                                                                                                                                          "   ExpiresDefault A86400\n"
                                                                                                                                                                                                                        ),
                                                                                                                                                                                                                        "   ExpiresByType application/octet-stream A300\n"
                                                                                                                                                                                                                      ),
                                                                                                                                                                                                                      "   ExpiresByType application/x-rpm A300\n"
                                                                                                                                                                                                                    ),
                                                                                                                                                                                                                    "</Directory>\n\n\n"
                                                                                                                                                                                                                  ),
                                                                                                                                                                                                                  "##\n"
                                                                                                                                                                                                                ),
                                                                                                                                                                                                                "## SSL Virtual Host Context\n"
                                                                                                                                                                                                              ),
                                                                                                                                                                                                              "##\n\n"
                                                                                                                                                                                                            ),
                                                                                                                                                                                                            "<VirtualHost _default_:631>\n\n"
                                                                                                                                                                                                          ),
                                                                                                                                                                                                          "    ServerName "
                                                                                                                                                                                                        ),
                                                                                                                                                                                                        Ops.get_string(
                                                                                                                                                                                                          primary_server,
                                                                                                                                                                                                          "CONFIG_LDAP_SERVER_IP",
                                                                                                                                                                                                          ""
                                                                                                                                                                                                        )
                                                                                                                                                                                                      ),
                                                                                                                                                                                                      ":631\n"
                                                                                                                                                                                                    ),
                                                                                                                                                                                                    "    DocumentRoot \"/srv/www/htdocs\"\n"
                                                                                                                                                                                                  ),
                                                                                                                                                                                                  "    ServerAdmin you@your.address\n"
                                                                                                                                                                                                ),
                                                                                                                                                                                                "    ErrorLog /var/log/apache2/error_log\n"
                                                                                                                                                                                              ),
                                                                                                                                                                                              "    TransferLog /var/log/apache2/access_log\n"
                                                                                                                                                                                            ),
                                                                                                                                                                                            "\t #Allow fully escaped URIs\n"
                                                                                                                                                                                          ),
                                                                                                                                                                                          "     AllowEncodedSlashes On\n\n"
                                                                                                                                                                                        ),
                                                                                                                                                                                        "    <Directory \"/srv/www/htdocs\">\n"
                                                                                                                                                                                      ),
                                                                                                                                                                                      "        Options Indexes FollowSymLinks\n"
                                                                                                                                                                                    ),
                                                                                                                                                                                    "        AllowOverride None\n"
                                                                                                                                                                                  ),
                                                                                                                                                                                  "        Require all granted\n"
                                                                                                                                                                                ),
                                                                                                                                                                              "    </Directory>\n\n"
                                                                                                                                                                            ),
                                                                                                                                                                            "    SSLEngine Optional\n"
                                                                                                                                                                          ),
                                                                                                                                                                          "    SSLProtocol all -SSLv2 -SSLv3\n"
                                                                                                                                                                        ),
                                                                                                                                                                        "    SSLCipherSuite ALL:!aNULL:!eNULL:!SSLv2:!LOW:!EXP:!MD5:@STRENGTH\n"
                                                                                                                                                                      ),
                                                                                                                                                                      "    #SSLCipherSuite ALL:!ADH:!EXPORT56:RC4+RSA:+HIGH:+MEDIUM:+LOW:+SSLv2:+EXP:+eNULL\n"
                                                                                                                                                                    ),
                                                                                                                                                                    "    SSLCertificateFile /etc/ssl/servercerts/servercert.pem\n"
                                                                                                                                                                  ),
                                                                                                                                                                  "    SSLCertificateKeyFile /etc/ssl/servercerts/serverkey.pem\n\n"
                                                                                                                                                                ),
                                                                                                                                                                "    #   SSL Engine Options:\n"
                                                                                                                                                              ),
                                                                                                                                                              "    #   Set various options for the SSL engine.\n"
                                                                                                                                                            ),
                                                                                                                                                            "    #   o FakeBasicAuth:\n"
                                                                                                                                                          ),
                                                                                                                                                          "    #     Translate the client X.509 into a Basic Authorisation.  This means that\n"
                                                                                                                                                        ),
                                                                                                                                                        "    #     the standard Auth/DBMAuth methods can be used for access control.  The\n"
                                                                                                                                                      ),
                                                                                                                                                      "    #     user name is the `one line' version of the client's X.509 certificate.\n"
                                                                                                                                                    ),
                                                                                                                                                    "    #     Note that no password is obtained from the user. Every entry in the user\n"
                                                                                                                                                  ),
                                                                                                                                                  "    #     file needs this password: `xxj31ZMTZzkVA'.\n"
                                                                                                                                                ),
                                                                                                                                                "    #   o ExportCertData:\n"
                                                                                                                                              ),
                                                                                                                                              "    #     This exports two additional environment variables: SSL_CLIENT_CERT and\n"
                                                                                                                                            ),
                                                                                                                                            "    #     SSL_SERVER_CERT. These contain the PEM-encoded certificates of the\n"
                                                                                                                                          ),
                                                                                                                                          "    #     server (always existing) and the client (only existing when client\n"
                                                                                                                                        ),
                                                                                                                                        "    #     authentication is used). This can be used to import the certificates\n"
                                                                                                                                      ),
                                                                                                                                      "    #     into CGI scripts.\n"
                                                                                                                                    ),
                                                                                                                                    "    #   o StdEnvVars:\n"
                                                                                                                                  ),
                                                                                                                                  "    #     This exports the standard SSL/TLS related `SSL_*' environment variables.\n"
                                                                                                                                ),
                                                                                                                                "    #     Per default this exportation is switched off for performance reasons,\n"
                                                                                                                              ),
                                                                                                                              "    #     because the extraction step is an expensive operation and is usually\n"
                                                                                                                            ),
                                                                                                                            "    #     useless for serving static content. So one usually enables the\n"
                                                                                                                          ),
                                                                                                                          "    #     exportation for CGI and SSI requests only.\n"
                                                                                                                        ),
                                                                                                                        "    #   o CompatEnvVars:\n"
                                                                                                                      ),
                                                                                                                      "    #     This exports obsolete environment variables for backward compatibility\n"
                                                                                                                    ),
                                                                                                                    "    #     to Apache-SSL 1.x, mod_ssl 2.0.x, Sioux 1.0 and Stronghold 2.x. Use this\n"
                                                                                                                  ),
                                                                                                                  "    #     to provide compatibility to existing CGI scripts.\n"
                                                                                                                ),
                                                                                                                "    #   o StrictRequire:\n"
                                                                                                              ),
                                                                                                              "    #     This denies access when \"SSLRequireSSL\" or \"SSLRequire\" applied even\n"
                                                                                                            ),
                                                                                                            "    #     under a \"Satisfy any\" situation, i.e. when it applies access is denied\n"
                                                                                                          ),
                                                                                                          "    #     and no other module can change it.\n"
                                                                                                        ),
                                                                                                        "    #   o OptRenegotiate:\n"
                                                                                                      ),
                                                                                                      "    #     This enables optimized SSL connection renegotiation handling when SSL\n"
                                                                                                    ),
                                                                                                    "    #     directives are used in per-directory context.\n"
                                                                                                  ),
                                                                                                  "    #SSLOptions +FakeBasicAuth +ExportCertData +CompatEnvVars +StrictRequire\n"
                                                                                                ),
                                                                                                "    <Files ~ \"\\.(cgi|shtml|phtml|php3?)$\">\n"
                                                                                              ),
                                                                                              "        SSLOptions +StdEnvVars\n"
                                                                                            ),
                                                                                            "    </Files>\n"
                                                                                          ),
                                                                                          "    <Directory \"/opt/novell/httpd/cgi-bin\">\n"
                                                                                        ),
                                                                                        "        SSLOptions +StdEnvVars\n"
                                                                                      ),
                                                                                      "    </Directory>\n\n"
                                                                                    ),
                                                                                    "    #   SSL Protocol Adjustments:\n"
                                                                                  ),
                                                                                  "    #   The safe and default but still SSL/TLS standard compliant shutdown\n"
                                                                                ),
                                                                                "    #   approach is that mod_ssl sends the close notify alert but doesn't wait for\n"
                                                                              ),
                                                                              "    #   the close notify alert from client. When you need a different shutdown\n"
                                                                            ),
                                                                            "    #   approach you can use one of the following variables:\n"
                                                                          ),
                                                                          "    #   o ssl-unclean-shutdown:\n"
                                                                        ),
                                                                        "    #     This forces an unclean shutdown when the connection is closed, i.e. no\n"
                                                                      ),
                                                                      "    #     SSL close notify alert is send or allowed to received.  This violates\n"
                                                                    ),
                                                                    "    #     the SSL/TLS standard but is needed for some brain-dead browsers. Use\n"
                                                                  ),
                                                                  "    #     this when you receive I/O errors because of the standard approach where\n"
                                                                ),
                                                                "    #     mod_ssl sends the close notify alert.\n"
                                                              ),
                                                              "    #   o ssl-accurate-shutdown:\n"
                                                            ),
                                                            "    #     This forces an accurate shutdown when the connection is closed, i.e. a\n"
                                                          ),
                                                          "    #     SSL close notify alert is send and mod_ssl waits for the close notify\n"
                                                        ),
                                                        "    #     alert of the client. This is 100% SSL/TLS standard compliant, but in\n"
                                                      ),
                                                      "    #     practice often causes hanging connections with brain-dead browsers. Use\n"
                                                    ),
                                                    "    #     this only for browsers where you know that their SSL implementation\n"
                                                  ),
                                                  "    #     works correctly.\n"
                                                ),
                                                "    #   Notice: Most problems of broken clients are also related to the HTTP\n"
                                              ),
                                              "    #   keep-alive facility, so you usually additionally want to disable\n"
                                            ),
                                            "    #   keep-alive for those clients, too. Use variable \"nokeepalive\" for this.\n"
                                          ),
                                          "    #   Similarly, one has to force some clients to use HTTP/1.0 to workaround\n"
                                        ),
                                        "    #   their broken HTTP/1.1 implementation. Use variables \"downgrade-1.0\" and\n"
                                      ),
                                      "    #   \"force-response-1.0\" for this.\n"
                                    ),
                                    "    SetEnvIf User-Agent \".*MSIE.*\" \\\n"
                                  ),
                                  "             nokeepalive ssl-unclean-shutdown \\\n"
                                ),
                                "             downgrade-1.0 force-response-1.0\n\n"
                              ),
                              "    CustomLog /var/log/apache2/ssl_request_log \\\n"
                            ),
                            "              \"%t %h %{SSL_PROTOCOL}x %{SSL_CIPHER}x \\\"%r\\\" %b\"\n\n"
                          ),
                          "# FIXME:  Our iprint_ssl.conf is already included from the following:\n"
                        ),
                        "#        /etc/apache2/httpd.conf\n"
                      ),
                      "#          |\n"
                    ),
                    "# Includes \\ /etc/apache2/vhosts.d/vhost-ssl.conf\n"
                  ),
                  "#              |\n"
                ),
                "#   Includes    \\ /etc/opt/novell/httpd/sslconf.d/*.conf\n"
              ),
              "#\n"
            ),
            "#Include /etc/opt/novell/httpd/sslconf.d/*.conf\n\n"
          ),
          "</VirtualHost>\n\n"
        )


        # Start out with a blank file.
        ret = SCR.Write(
          path(".target.string"),
          "/etc/opt/novell/iprint/httpd/conf/iprint_g.conf",
          ""
        )
        ret = SCR.Write(
          path(".target.string"),
          "/etc/opt/novell/iprint/httpd/conf/iprint_g.conf",
          iprint_g_conf
        )
        if ret != true
          Report.Warning(
            _(
              "Failed to write to /etc/opt/novell/iprint/httpd/conf/iprint_g.conf"
            )
          )
        end
      else
        Builtins.y2milestone("Iprint.write:skipping config for upgrade.")
        extend_schema()
      end

      #  Disable cups #720832
      cmd = "/usr/sbin/rccups stop"
      Builtins.y2milestone("iPrint:Write stop rccups command: %1", cmd)
      mymap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(mymap, "exit", -1) != 0
        Builtins.y2milestone("iPrint:Write stop rccups failed: %1 using", mymap)
      end

      cmd = "/sbin/insserv -r cups"
      Builtins.y2milestone("iPrint:Write disable rccups command: %1", cmd)
      mymap = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      if Ops.get_integer(mymap, "exit", -1) != 0
        Builtins.y2milestone(
          "iPrint:Write disable rccups failed: %1 using",
          mymap
        )
      end

      # Configure the firewall ports
      NovellUtils.SetFirewallPort(80, :tcp, true)
      NovellUtils.SetFirewallPort(443, :tcp, true)
      NovellUtils.SetFirewallPort(631, :tcp, true)

      # bug 1057390
      if NovellPatterns.IsPatternInstalled("iprint-advanced")
        cmd2 = "/opt/novell/iprintmobile/bin/ipp_page_enable_disable.sh --init"
        Builtins.y2milestone("Executing: %1", cmd2)
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
        Builtins.y2milestone(
          "Executed ipp_page_enable_disable.sh:%1",
          val
        )
      end

      if Mode.update || Mode.autoupgrade
        Builtins.y2milestone(
          "iprint.write:restarting iprint services in upgrade mode."
        )

        if @upgrade_from == "OES2015SP1" && NovellPatterns.IsPatternInstalled("iprint-advanced")
          cmd2 = "/opt/novell/iprint/bin/post_update_fix"
          Builtins.y2milestone("Executing: %1", cmd2)
          val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
          Builtins.y2milestone(
            "Executed post_update_fix.sh:%1",
            val
          )
        end

        cmd2 = "/usr/sbin/rcnovell-ipsmd restart"
        Builtins.y2milestone("Executing: %1", cmd2)
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
        exit = Ops.get_integer(val, "exit", -1)
        Builtins.y2milestone(
          "iprint.service restart of ipsmd:return code:%1",
          exit
        )

        cmd2 = "/usr/sbin/rcnovell-idsd restart"
        Builtins.y2milestone("Executing: %1", cmd2)
        val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd2))
        exit = Ops.get_integer(val, "exit", -1)
        Builtins.y2milestone(
          "iprint.service restart of idsd:return code:%1",
          exit
        ) 
		
        if NovellPatterns.IsPatternInstalled("iprint-advanced")
          if Misc.CustomSysconfigRead("mobility_addon_configured", "", "/etc/opt/novell/iprint/conf/iprintconf.properties") == "true"
            if !Service.Enabled("iprint-auth")
              Service.Enable("iprint-auth")
            end
            Service.Start("iprint-auth")
            services_list = ["novell-iprint-license", "novell-iprint-tomcat", "novell-iprint-ocs", "orientdb"]
            services_list.each do |service|
              if Service.Enabled(service)
                Builtins.y2milestone(("%1 service is enabled"),service)
                Service.Restart(service)
                  if Service.Active(service)
                    Builtins.y2milestone(("%1 service is running"),service)
                  else
                    Builtins.y2milestone(("%1 service is not running"),service)
                  end
              else
                Builtins.y2milestone(("%1 is disabled"),service)
              end
            end
          else
            Builtins.y2milestone("Value of mobility_addon_configured in /etc/opt/novell/iprint/conf/iprintconf.properties is not true")		  
          end
        else
          Builtins.y2milestone("iprint-advanced pattern is not installed")
        end

        #        cmd = "/opt/novell/iprint/bin/iprint_clean_restart";
        #	y2milestone("Executing: %1", cmd);
        #        val = (map)SCR::Execute(.target.bash_output, cmd);
        #        exit = val["exit"]:-1;
        #	string stdout = val["stdout"]:"";
        #        y2milestone("iprint.call to iprint_clean_restart:return code:%1, stdout:%2", exit, stdout );
      else
        # Start the iPrint services.
        Service.Enable("novell-idsd")
        Service.Enable("novell-ipsmd")
        Service.Start("novell-idsd")
        Service.Start("novell-ipsmd")
        if NovellPatterns.IsPatternInstalled("iprint-advanced")
          Service.Restart("novell-iprint-tomcat")
          if Service.Active("novell-iprint-tomcat")
            Builtins.y2milestone("novell-iprint-tomcat service is running")
          else
            Builtins.y2milestone("novell-iprint-tomcat service is not running")
          end
        end
      end

      SCR.Write(path(".sysconfig.novell-common.START_APACHE_TOMCAT"), "yes")


      # Write the /etc/sysconfig file
      SCR.Write(
        path(".sysconfig.iprint.CONFIG_IPRINT_LDAP_SERVER"),
        Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", "")
      )
      SCR.Write(
        path(".sysconfig.iprint.CONFIG_IPRINT_TOP_CONTEXT"),
        Ops.get_string(@Settings, "CONFIG_IPRINT_TOP_CONTEXT", "")
      )
      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.iprint"), nil)

      # Write and Flush a marker file for the inst_oes_congratulate kludge.
      SCR.Write(path(".target.string"), "/opt/novell/oes-install/ip", "k1")
      SCR.Write(path(".target.string"), "/opt/novell/oes-install/ip", nil)

      Builtins.y2debug("iPrint Settings(Write): %1", @Settings)

      SCR.Write(path(".sysconfig.iprint.SERVICE_CONFIGURED"), "yes")

      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.iprint"), nil)

      return false if Abort()
      Progress.Finish
      Builtins.sleep(sl)

      return false if Abort()
      true
    end

    # Get all iprint settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      if settings == {}
        @globals_configured = false
        Ops.set(NovellAddonInstall.products_to_configure, "iprint", false)
        return true
      end

      Ops.set(
        @Settings,
        "CONFIG_IPRINT_LDAP_SERVER",
        Ops.get_string(settings, "ldap_server", "")
      )
      Ops.set(
        @Settings,
        "CONFIG_IPRINT_TOP_CONTEXT",
        Ops.get_string(settings, "top_context", "")
      )

      @globals_configured = true
      true
    end

    # Dump the iprint settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      settings = {}

      return settings if NovellUtils.CloneSettingstoAutoinstXml("novell-iprint") == false

      if @auto_clone
        Ops.set(NovellAddonInstall.products_to_configure, "iprint", true)
      end
      if Ops.get(NovellAddonInstall.products_to_configure, "iprint", true)
        Ops.set(
          settings,
          "ldap_server",
          Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", "")
        )
        Ops.set(
          settings,
          "top_context",
          Ops.get_string(@Settings, "CONFIG_IPRINT_TOP_CONTEXT", "")
        )
      end
      deep_copy(settings)
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      summary = ""

      if !@globals_configured
        nc = Summary.NotConfigured
        summary = Summary.AddLine(summary, nc)
      else
        summary = Summary.OpenList(summary)

        summary = Summary.AddListItem(
          summary,
          Builtins.sformat(
            _("iPrint eDirectory tree: %1"),
            Ops.get_string(NovellLDAP.Settings, "CONFIG_LDAP_TREE_NAME", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("LDAP Server: %1"),
            Ops.get_string(@Settings, "CONFIG_IPRINT_LDAP_SERVER", "")
          )
        )

        summary = Summary.AddListItem(
          summary, # Configuration summary text for autoyast
          Builtins.sformat(
            _("eDirectory search context: %1"),
            Ops.get_string(@Settings, "CONFIG_IPRINT_TOP_CONTEXT", "")
          )
        )

        summary = Summary.CloseList(summary)
      end
      [summary, []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :Settings, :type => "map"
    publish :variable => :oldSettings, :type => "map"
    publish :variable => :auto_clone, :type => "boolean"
    publish :variable => :xad_packages_installed, :type => "boolean"
    publish :variable => :xadMode, :type => "boolean"
    publish :variable => :xadTestMode, :type => "boolean"
    publish :variable => :edir_service_configured, :type => "boolean"
    publish :variable => :schemaLogPath, :type => "string"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :globals_configured, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  Iprint = IprintClass.new
  Iprint.main
end
